/* 
   Problema Manutencao -- manut.c
   Seletiva IOI 2003 
   Guilherme Ottoni
   Algoritmo: busca em profundidade adaptada.
              complexidade: O(n^2)
*/

#include <stdio.h>
#include <string.h>

#define MAX_COMPUTADORES 400
#define MIN(X,Y) ((X) < (Y) ? (X) : (Y))

#define BRANCO 0
#define CINZA  1
#define PRETO  2

char grafo[MAX_COMPUTADORES][MAX_COMPUTADORES];
char cor[MAX_COMPUTADORES];
int nivel[MAX_COMPUTADORES];
char articulacao[MAX_COMPUTADORES];
int n;

/* busca em profundidade modificada, que retorna o menor nivel que pode 
   ser alcancado na sub-arvore por uma aresta de retorno (que vai para um 
   vertice ja visitado) */
int bp(int i, int niv)
{
  int min_nivel=niv, j, nfilhos=0, min_nivel_filho;

  nivel[i] = niv;
  cor[i] = CINZA;
  for (j = 0; j < n; j++)
    if (grafo[i][j])
      {
	if (cor[j] == CINZA) /* aresta de retorno */
	  min_nivel = MIN(min_nivel, nivel[j]);
	else 
	  if (cor[j] == BRANCO) /* aresta de arvore */
	    {
	      min_nivel_filho = bp(j, niv + 1);
	      /* se i nao eh a raiz e tem um filho que nao alcanca um nivel 
		 menor que niv, entao i eh articulacao */
	      if (niv > 0 && min_nivel_filho >= niv)
		articulacao[i] = 1;
	      min_nivel = MIN(min_nivel, min_nivel_filho);
	      nfilhos++;
	    }
      }
  /* se i eh a raiz e tem mais de 1 filho, entao i eh articulacao */
  if (niv == 0 && nfilhos > 1)
    articulacao[i] = 1;
  cor[i] = PRETO;
  return min_nivel;
}

int main()
{
  int teste=1, x, y, nenhum, m;
  while (scanf("%d %d", &n, &m) == 2 && n > 0)
    {
      memset(grafo, 0, sizeof(grafo));
      memset(cor, BRANCO, sizeof(cor));
      memset(articulacao, 0, sizeof(articulacao));
      while (m--)
	{
	  scanf("%d %d", &x, &y);
	  grafo[x-1][y-1] = grafo[y-1][x-1] = 1;	  
	}
      bp(0, 0);
      printf("Teste %d\n", teste++);
      nenhum = 1;
      for (x = 0; x < n; x++)
	if (articulacao[x])
	  {
	    printf("%d ", x + 1);
	    nenhum = 0;
	  }
      printf("%s\n\n", nenhum ? "nenhum" : "");
    }
  return 0;
}
